@@ -25,12 +25,13 @@ import (
 
 	"golang.org/x/sync/errgroup"
 	"golang.org/x/tools/go/types/objectpath"
+	"golang.org/x/tools/go/types/typeutil"
 	"golang.org/x/tools/gopls/internal/cache"
 	"golang.org/x/tools/gopls/internal/cache/metadata"
 	"golang.org/x/tools/gopls/internal/cache/methodsets"
+	"golang.org/x/tools/gopls/internal/cache/parsego"
 	"golang.org/x/tools/gopls/internal/file"
 	"golang.org/x/tools/gopls/internal/protocol"
-	"golang.org/x/tools/gopls/internal/util/bug"
 	"golang.org/x/tools/gopls/internal/util/safetoken"
 	"golang.org/x/tools/internal/event"
 )
@@ -164,7 +165,7 @@ func packageReferences(ctx context.Context, snapshot *cache.Snapshot, uri protoc
 				if err != nil {
 					return nil, err
 				}
-				f, err := snapshot.ParseGo(ctx, fh, ParseHeader)
+				f, err := snapshot.ParseGo(ctx, fh, parsego.Header)
 				if err != nil {
 					return nil, err
 				}
@@ -193,7 +194,7 @@ func packageReferences(ctx context.Context, snapshot *cache.Snapshot, uri protoc
 		if err != nil {
 			return nil, err
 		}
-		f, err := snapshot.ParseGo(ctx, fh, ParseHeader)
+		f, err := snapshot.ParseGo(ctx, fh, parsego.Header)
 		if err != nil {
 			return nil, err
 		}
@@ -234,7 +235,7 @@ func ordinaryReferences(ctx context.Context, snapshot *cache.Snapshot, uri proto
 	if err != nil {
 		return nil, err
 	}
-	candidates, _, err := objectsAt(pkg.GetTypesInfo(), pgf.File, pos)
+	candidates, _, err := objectsAt(pkg.TypesInfo(), pgf.File, pos)
 	if err != nil {
 		return nil, err
 	}
@@ -251,15 +252,9 @@ func ordinaryReferences(ctx context.Context, snapshot *cache.Snapshot, uri proto
 	}
 
 	// nil, error, error.Error, iota, or other built-in?
-	if obj.Pkg() == nil {
+	if isBuiltin(obj) {
 		return nil, fmt.Errorf("references to builtin %q are not supported", obj.Name())
 	}
-	if !obj.Pos().IsValid() {
-		if obj.Pkg().Path() != "unsafe" {
-			bug.Reportf("references: object %v has no position", obj)
-		}
-		return nil, fmt.Errorf("references to unsafe.%s are not supported", obj.Name())
-	}
 
 	// Find metadata of all packages containing the object's defining file.
 	// This may include the query pkg, and possibly other variants.
@@ -421,7 +416,7 @@ func ordinaryReferences(ctx context.Context, snapshot *cache.Snapshot, uri proto
 			if err != nil {
 				return err
 			}
-			objects, _, err := objectsAt(pkg.GetTypesInfo(), pgf.File, pos)
+			objects, _, err := objectsAt(pkg.TypesInfo(), pgf.File, pos)
 			if err != nil {
 				return err // unreachable? (probably caught earlier)
 			}
@@ -457,7 +452,7 @@ func ordinaryReferences(ctx context.Context, snapshot *cache.Snapshot, uri proto
 
 			targets := make(map[types.Object]bool)
 			for objpath := range globalTargets[pkg.Metadata().PkgPath] {
-				obj, err := objectpath.Object(pkg.GetTypes(), objpath)
+				obj, err := objectpath.Object(pkg.Types(), objpath)
 				if err != nil {
 					// No such object, because it was
 					// declared only in the test variant.
@@ -513,7 +508,10 @@ func expandMethodSearch(ctx context.Context, snapshot *cache.Snapshot, workspace
 	// Compute the method-set fingerprint used as a key to the global search.
 	key, hasMethods := methodsets.KeyOf(recv)
 	if !hasMethods {
-		return bug.Errorf("KeyOf(%s)={} yet %s is a method", recv, method)
+		// The query object was method T.m, but methodset(T)={}:
+		// this indicates that ill-typed T has conflicting fields and methods.
+		// Rather than bug-report (#67978), treat the empty method set at face value.
+		return nil
 	}
 	// Search the methodset index of each package in the workspace.
 	indexes, err := snapshot.MethodSets(ctx, workspaceIDs...)
@@ -527,7 +525,7 @@ func expandMethodSearch(ctx context.Context, snapshot *cache.Snapshot, workspace
 		index := index
 		group.Go(func() error {
 			// Consult index for matching methods.
-			results := index.Search(key, method.Name())
+			results := index.Search(key, method)
 			if len(results) == 0 {
 				return nil
 			}
@@ -582,6 +580,8 @@ func localReferences(pkg *cache.Package, targets map[types.Object]bool, correspo
 		}
 	}
 
+	var msets typeutil.MethodSetCache
+
 	// matches reports whether obj either is or corresponds to a target.
 	// (Correspondence is defined as usual for interface methods.)
 	matches := func(obj types.Object) bool {
@@ -591,7 +591,7 @@ func localReferences(pkg *cache.Package, targets map[types.Object]bool, correspo
 		if methodRecvs != nil && obj.Name() == methodName {
 			if orecv := effectiveReceiver(obj); orecv != nil {
 				for _, mrecv := range methodRecvs {
-					if concreteImplementsIntf(orecv, mrecv) {
+					if concreteImplementsIntf(&msets, orecv, mrecv) {
 						return true
 					}
 				}
@@ -604,7 +604,7 @@ func localReferences(pkg *cache.Package, targets map[types.Object]bool, correspo
 	for _, pgf := range pkg.CompiledGoFiles() {
 		ast.Inspect(pgf.File, func(n ast.Node) bool {
 			if id, ok := n.(*ast.Ident); ok {
-				if obj, ok := pkg.GetTypesInfo().Uses[id]; ok && matches(obj) {
+				if obj, ok := pkg.TypesInfo().Uses[id]; ok && matches(obj) {
 					report(mustLocation(pgf, id), false)
 				}
 			}
@@ -618,7 +618,7 @@ func localReferences(pkg *cache.Package, targets map[types.Object]bool, correspo
 // comparisons for obj, if it is a method, or nil otherwise.
 func effectiveReceiver(obj types.Object) types.Type {
 	if fn, ok := obj.(*types.Func); ok {
-		if recv := fn.Type().(*types.Signature).Recv(); recv != nil {
+		if recv := fn.Signature().Recv(); recv != nil {
 			return methodsets.EnsurePointer(recv.Type())
 		}
 	}
@@ -656,12 +656,6 @@ func objectsAt(info *types.Info, file *ast.File, pos token.Pos) (map[types.Objec
 				targets[obj] = leaf
 			}
 		} else {
-			// Note: prior to go1.21, go/types issue #60372 causes the position
-			// a field Var T created for struct{*p.T} to be recorded at the
-			// start of the field type ("*") not the location of the T.
-			// This affects references and other gopls operations (issue #60369).
-			// TODO(adonovan): delete this comment when we drop support for go1.20.
-
 			// For struct{T}, we prefer the defined field Var over the used TypeName.
 			obj := info.ObjectOf(leaf)
 			if obj == nil {
@@ -688,7 +682,7 @@ func objectsAt(info *types.Info, file *ast.File, pos token.Pos) (map[types.Objec
 // which must belong to m.File.
 //
 // Safe for use only by references and implementations.
-func mustLocation(pgf *ParsedGoFile, n ast.Node) protocol.Location {
+func mustLocation(pgf *parsego.File, n ast.Node) protocol.Location {
 	loc, err := pgf.NodeLocation(n)
 	if err != nil {
 		panic(err) // can't happen in references or implementations
